@@ -1,7 +1,7 @@
 '''
 HTTP stuff.
 
-Copyright (c) 2015 - 2022 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
+Copyright (c) 2015 - 2024 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -31,6 +31,7 @@ import requests
 from tqdm import tqdm
 
 from buildtools.bt_logging import IndentLogger
+from buildtools.types import StrOrPath
 
 HTTP_METHOD_GET = 'GET'
 HTTP_METHOD_POST = 'POST'
@@ -38,7 +39,7 @@ HTTP_METHOD_POST = 'POST'
 log = IndentLogger(logging.getLogger(__name__))
 
 
-def DownloadFile(url: str, filename: Union[str, Path], log_after: bool = True, print_status: bool = True, log_before: bool = True, **kwargs) -> bool:
+def DownloadFile(url: str, filename: StrOrPath, log_after: bool = True, print_status: bool = True, log_before: bool = True, **kwargs) -> bool:
     '''
     Download a file from url to filename.
 
@@ -107,3 +108,4 @@ def DownloadFile(url: str, filename: Union[str, Path], log_after: bool = True, p
             progress.close()
     if log_after:
         log.info(f'Downloaded {url} to {output} ({file_size_dl}B)')
+    return True
