@@ -1,7 +1,7 @@
 '''
 File indentation stuff for my OCD :V
 
-Copyright (c) 2015 - 2022 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
+Copyright (c) 2015 - 2024 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,7 @@ from __future__ import annotations
 
 import logging
 import re
-from typing import Collection, Dict, List, Optional, Pattern, Sequence, TextIO, Tuple
+from typing import Dict, List, Optional, Pattern, Sequence, TextIO, Tuple
 
 from buildtools.bt_logging import IndentLogger
 
