@@ -1,7 +1,7 @@
-'''
+"""
 Maestro is my crappy, synchronous, yet extensible build system
 
-Copyright (c) 2015 - 2022 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
+Copyright (c) 2015 - 2024 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -20,16 +20,17 @@ AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
-'''
+"""
 
 import argparse
 import codecs
 import logging
 import os
+from pathlib import Path
 import shutil
 import sys
 import traceback
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Union
 
 from ruamel.yaml import YAML
 
@@ -38,7 +39,7 @@ from buildtools.bt_logging import log
 from buildtools.maestro.base_target import BuildTarget
 from buildtools.maestro.utils import SerializableFileLambda, SerializableLambda
 
-yaml = YAML(typ='safe', pure=True)
+yaml = YAML(typ="safe", pure=True)
 
 yaml.register_class(SerializableLambda)
 yaml.register_class(SerializableFileLambda)
@@ -96,7 +97,7 @@ class TarjanGraph:
 class BuildMaestro:
     # ALL_TYPES = {}
 
-    def __init__(self, hidden_build_dir='.build'):
+    def __init__(self, hidden_build_dir: Union[str, Path] = ".build"):
         self.alltargets: List[BuildTarget] = []
         self.targets: List[str] = []
         self.targetsCompleted: List[str] = []
@@ -106,8 +107,9 @@ class BuildMaestro:
         self.colors: bool = False
         self.show_commands: bool = False
 
-        self.builddir: str = hidden_build_dir
-        self.all_targets_file: str = os.path.join(self.builddir, 'all_targets.yml')
+        self.builddir: Path = Path(hidden_build_dir)
+        self.cache_dir:Path = self.builddir / "cache"
+        self.all_targets_file: Path = self.builddir / "all_targets.yml"
 
         # This will get Maestro to delete the listed directories, if they are present during --clean.
         self.other_dirs_to_clean: List[str] = []
@@ -120,31 +122,50 @@ class BuildMaestro:
         self.alltargets.append(bt)
         for t in bt.provides():
             if t in self.targets:
-                log.warn(f'Target {t} already exists.')
+                log.warn(f"Target {t} already exists.")
                 old = self.targetsProvidedBy[t]
-                with log.warn('Old:'):
+                with log.warn("Old:"):
                     old._dumpStacks()
-                with log.warn('New:'):
+                with log.warn("New:"):
                     bt._dumpStacks()
             self.targetsProvidedBy[t] = bt
             self.targets.append(t)
         return bt
 
     def generateVirtualTarget(self, id: str) -> str:
-        '''
+        """
         Usage:
 
         bm.add(CopyFilesTarget(target=bm.generateVirtualTarget('a'),...))
-        '''
-        return os.path.join(self.builddir, '.targets', id)
+        """
+        return str(self.builddir / ".targets" / id)
 
     def build_argparser(self):
         argp = argparse.ArgumentParser()
-        argp.add_argument('--clean', action='store_true', default=False, help='Cleans everything.')
-        argp.add_argument('--no-colors', action='store_true', default=False, help='Disables colors.')
-        argp.add_argument('--rebuild', action='store_true', default=False, help='Clean rebuild of project.')
-        argp.add_argument('--show-commands', action='store_true', default=False, help='Echoes the line used to execute commands. (echo=True in os_utils.cmd())')
-        argp.add_argument('--verbose', action='store_true', default=False, help='Show hidden buildsteps.')
+        argp.add_argument(
+            "--clean", action="store_true", default=False, help="Cleans everything."
+        )
+        argp.add_argument(
+            "--no-colors", action="store_true", default=False, help="Disables colors."
+        )
+        argp.add_argument(
+            "--rebuild",
+            action="store_true",
+            default=False,
+            help="Clean rebuild of project.",
+        )
+        argp.add_argument(
+            "--show-commands",
+            action="store_true",
+            default=False,
+            help="Echoes the line used to execute commands. (echo=True in os_utils.cmd())",
+        )
+        argp.add_argument(
+            "--verbose",
+            action="store_true",
+            default=False,
+            help="Show hidden buildsteps.",
+        )
         return argp
 
     def parse_args(self, argp=None, args=None):
@@ -172,10 +193,13 @@ class BuildMaestro:
 
     def clean(self):
         older_files = set()
-        if os.path.isfile(self.all_targets_file):
-            with open(self.all_targets_file, 'r', encoding='utf-8') as f:
+        if self.all_targets_file.is_file():
+            with open(self.all_targets_file, "r", encoding="utf-8") as f:
                 try:
-                    older_files = set(sorted(yaml.load(f)))
+                    d = yaml.load(f)
+                    if hasattr(d, "get"):
+                        d = d.get("targets")
+                    older_files = set(sorted(d))
                 except BaseException:
                     older_files = []
         for bt in self.alltargets:
@@ -184,29 +208,36 @@ class BuildMaestro:
         for xtradir in self.other_dirs_to_clean:
             if os.path.isdir(xtradir):
                 if self.colors:
-                    log.info('<red>RMTREE</red> %s <red>(other_dirs_to_clean)</red>', xtradir)
+                    log.info(
+                        "<red>RMTREE</red> %s <red>(other_dirs_to_clean)</red>", xtradir
+                    )
                 else:
-                    log.info('RMTREE %s (other_dirs_to_clean)', xtradir)
+                    log.info("RMTREE %s (other_dirs_to_clean)", xtradir)
                 shutil.rmtree(xtradir, ignore_errors=True)
-        if os.path.isdir(self.builddir):
+
+        if self.builddir.is_file():
             if self.colors:
-                log.info('<red>RMTREE</red> %s <red>(build system stuff)</red>', self.builddir)
+                log.info(
+                    "<red>RMTREE</red> %s <red>(build system stuff)</red>",
+                    self.builddir,
+                )
             else:
-                log.info('RMTREE %s (build system stuff)', self.builddir)
+                log.info("RMTREE %s (build system stuff)", self.builddir)
             shutil.rmtree(self.builddir, ignore_errors=True)
-        with log.info('Cleaning up old unclaimed files...'):
+
+        with log.info("Cleaning up old unclaimed files..."):
             for targetfile in older_files:
                 targetfile = os.path.normpath(targetfile)
                 if os.path.isfile(targetfile):
                     if self.colors:
-                        log.info('<red>RM</red> %s', targetfile)
+                        log.info("<red>RM</red> %s", targetfile)
                     else:
-                        log.info('RM %s', targetfile)
+                        log.info("RM %s", targetfile)
                     os.remove(targetfile)
 
     @staticmethod
     def RecognizeType(cls):
-        #BuildMaestro.ALL_TYPES[cls.BT_TYPE] = cls
+        # BuildMaestro.ALL_TYPES[cls.BT_TYPE] = cls
         pass
 
     # def saveRules(self, filename):
@@ -288,7 +319,7 @@ class BuildMaestro:
         return 0
 
     def checkForCycles(self):
-        with log.info('Checking for dependency cycles...'):
+        with log.info("Checking for dependency cycles..."):
             # Using Tarjan's Strongly Connected Cycles algorithm
             tg = TarjanGraph()
 
@@ -297,21 +328,41 @@ class BuildMaestro:
                 refs = []
                 for depend in bt.dependencies:
                     if not isinstance(depend, str):
-                        log.critical('Build target %s has invalid dependency %s.', bt.name, depend)
+                        log.critical(
+                            "Build target %s has invalid dependency %s.",
+                            bt.name,
+                            depend,
+                        )
                         sys.exit(1)
                     providers = []
                     for obt in self.alltargets:
                         if depend in obt.provides():
-                            #log.info('%s provides %s, which %s needs', obt.name, depend, bt.name)
+                            # log.info('%s provides %s, which %s needs', obt.name, depend, bt.name)
                             providers += [obt]
                     if len(providers) > 1:
-                        log.warning('Build target %s has %d providers for dependency %s: %r', bt.name, len(providers), depend, [x.name for x in providers])
+                        log.warning(
+                            "Build target %s has %d providers for dependency %s: %r",
+                            bt.name,
+                            len(providers),
+                            depend,
+                            [x.name for x in providers],
+                        )
                     elif len(providers) == 0:
-                        log.critical('Build target %s has no providers for dependency %s: %r', bt.name, depend, [x.name for x in providers])
+                        log.critical(
+                            "Build target %s has no providers for dependency %s: %r",
+                            bt.name,
+                            depend,
+                            [x.name for x in providers],
+                        )
                         sys.exit(1)
                     refs.append(providers[-1].id)
-                with log.debug('Dependency tree:'):
-                    with log.debug('[%s] (%d,[%s])', bt.name, bt.id, ', '.join([str(x) for x in refs])):
+                with log.debug("Dependency tree:"):
+                    with log.debug(
+                        "[%s] (%d,[%s])",
+                        bt.name,
+                        bt.id,
+                        ", ".join([str(x) for x in refs]),
+                    ):
                         for refID in refs:
                             log.debug(self.alltargets[refID].name)
                 tg.add_edge(bt.id, refs)
@@ -323,7 +374,15 @@ class BuildMaestro:
             foundCycles = False
             for cycle in tg.cycles:
                 if len(cycle) > 1:
-                    log.critical('CYCLE FOUND: %r', ['#{} ({})'.format(self.alltargets[btid].id, self.alltargets[btid].name) for btid in cycle])
+                    log.critical(
+                        "CYCLE FOUND: %r",
+                        [
+                            "#{} ({})".format(
+                                self.alltargets[btid].id, self.alltargets[btid].name
+                            )
+                            for btid in cycle
+                        ],
+                    )
                     foundCycles = True
             return foundCycles
 
@@ -333,9 +392,9 @@ class BuildMaestro:
             if bt.built:
                 for targetfile in bt.provides():
                     alltargets.add(targetfile)
-        os_utils.ensureDirExists(os.path.dirname(self.all_targets_file))
-        with open(self.all_targets_file, 'w', encoding='utf-8') as f:
-            yaml.dump(list(alltargets), f)
+        self.all_targets_file.parent.mkdir(parents=True, exist_ok=True)
+        with open(self.all_targets_file, "w", encoding="utf-8") as f:
+            yaml.dump({"targets": list(alltargets)}, f)
 
     def run(self, verbose: Optional[bool] = None):
         if verbose is not None:
@@ -344,7 +403,7 @@ class BuildMaestro:
         new_targets = []
         for t in self.targets:
             if t in new_targets:
-                log.warn('Target %s added more than once.', t)
+                log.warn("Target %s added more than once.", t)
             else:
                 new_targets.append(t)
 
@@ -363,14 +422,16 @@ class BuildMaestro:
         #        if reqfile in keys and reqfile not in target.dependencies:
         #            target.dependencies.append(reqfile)
         loop = 0
-        #progress = tqdm(total=len(self.targets), unit='target', desc='Building', leave=False)
+        # progress = tqdm(total=len(self.targets), unit='target', desc='Building', leave=False)
         self.targetsCompleted = []
         self.targetsDirty = []
         while len(self.targets) > len(self.targetsCompleted) and loop < 100:
             loop += 1
             for bt in self.alltargets:
                 bt.maestro = self
-                if bt.canBuild(self, keys) and any([target not in self.targetsCompleted for target in bt.provides()]):
+                if bt.canBuild(self, keys) and any(
+                    [target not in self.targetsCompleted for target in bt.provides()]
+                ):
                     try:
                         bt.try_build()
                         # progress.update(1)
@@ -380,37 +441,50 @@ class BuildMaestro:
                     except Exception as e:
                         bt._set_failed()
                         self._write_targets()
-                        log.critical('An exception occurred, build halted.')
+                        log.critical("An exception occurred, build halted.")
                         log.exception(e)
                         return
                     except KeyboardInterrupt:
                         bt._set_failed()
                         self._write_targets()
-                        log.critical('Cancelled via KeyboardInterrupt.')
+                        log.critical("Cancelled via KeyboardInterrupt.")
                         return
                     bt.built = True
-            log.debug('%d > %d, loop = %d', len(self.targets), len(self.targetsCompleted), loop)
-        log.debug('%d > %d, loop = %d', len(self.targets), len(self.targetsCompleted), loop)
+            log.debug(
+                "%d > %d, loop = %d",
+                len(self.targets),
+                len(self.targetsCompleted),
+                loop,
+            )
+        log.debug(
+            "%d > %d, loop = %d", len(self.targets), len(self.targetsCompleted), loop
+        )
         # progress.close()
         self._write_targets()
         if loop >= 100:
-            incompleteTargets = [t for t in self.targets if t not in self.targetsCompleted]
+            incompleteTargets = [
+                t for t in self.targets if t not in self.targetsCompleted
+            ]
             if len(incompleteTargets) > 0:
-                with log.critical("Failed to resolve dependencies.  The following targets are left unresolved. Exiting."):
+                with log.critical(
+                    "Failed to resolve dependencies.  The following targets are left unresolved. Exiting."
+                ):
                     for t in incompleteTargets:
                         log.critical(t)
             orphanDeps = [t for t in alldeps if t not in self.targets]
             if len(orphanDeps) > 0:
-                with log.critical("Failed to resolve dependencies.  The following dependencies are orphaned. Exiting."):
+                with log.critical(
+                    "Failed to resolve dependencies.  The following dependencies are orphaned. Exiting."
+                ):
                     for t in orphanDeps:
                         log.critical(t)
             # sys.exit(1)
-        with log.info('Cleaning up...'):
+        with log.info("Cleaning up..."):
             cachefiles = []
             for bt in self.alltargets:
                 cachefiles.append(os.path.basename(bt.getCacheFile()))
-            for filename in os.listdir(os.path.join(self.builddir, 'cache')):
-                if filename not in cachefiles:
-                    filename = os.path.join(self.builddir, 'cache', filename)
-                    log.debug('<red>RM</red> %s', filename)
-                    os.remove(filename)
+            if self.cache_dir.is_dir():
+                for filename in self.cache_dir.iterdir():
+                    if filename not in cachefiles:
+                        log.debug("<red>RM</red> %s", filename)
+                        os.remove(filename)
