@@ -1,7 +1,7 @@
 '''
 Base BuildTarget
 
-Copyright (c) 2015 - 2022 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
+Copyright (c) 2015 - 2024 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,6 @@ import hashlib
 import os
 import traceback
 import typing
-from inspect import trace
 from pathlib import Path
 from traceback import StackSummary
 from typing import Any, Callable, Dict, List, Optional, Set, Union
@@ -39,7 +38,7 @@ from buildtools import os_utils, utils
 from buildtools.bt_logging import IndentLogger, log
 from buildtools.maestro.utils import callLambda
 
-yaml = YAML(typ='safe', pure=True)
+yaml = YAML(typ='rt')
 
 
 if typing.TYPE_CHECKING:
@@ -56,7 +55,7 @@ class BuildTarget(object):
     BT_LABEL: str = 'FIXME'
 
     #                YYYYMMDDhhmm
-    CACHE_VER: int = 202205141832
+    CACHE_VER: int = 202305041705
 
     CHECK_MTIMES: bool = True
     CHECK_HASHES: bool = True
@@ -227,7 +226,7 @@ class BuildTarget(object):
     def getCacheFile(self) -> str:
         assert self.maestro is not None
         filename = hashlib.md5(self.name.encode('utf-8')).hexdigest() + '.yml'
-        return os.path.join(self.maestro.builddir, 'cache', filename)
+        return os.path.join(self.maestro.cache_dir, filename)
 
     def getConfigHash(self) -> str:
         s = StringIO()
@@ -391,7 +390,7 @@ class BuildTarget(object):
         return False
 
     def canBuild(self, maestro: 'BuildMaestro', keys: List[str]) -> bool:
-        #self.files = list(callLambda(self.files))
+        # self.files = list(callLambda(self.files))
         # for dep in list(set(self.dependencies + self.files)):
         if not self._lambdas_called:
             for reqfile in callLambda(self.files):
