@@ -1,7 +1,7 @@
 '''
 BLURB GOES HERE.
 
-Copyright (c) 2015 - 2022 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
+Copyright (c) 2015 - 2024 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -24,13 +24,13 @@ SOFTWARE.
 '''
 import codecs
 import os
-from pathlib import Path
-from typing import List, Optional, Union
+from typing import List, Optional
 
 from tqdm import tqdm
 
 from buildtools import log, os_utils
 from buildtools.maestro.base_target import SingleBuildTarget
+from buildtools.types import StrOrPath
 
 
 class CoffeeBuildTarget(SingleBuildTarget):
@@ -43,9 +43,9 @@ class CoffeeBuildTarget(SingleBuildTarget):
                  files: List[str] = [],
                  dependencies: List[str] = [],
                  coffee_opts: List[str] = ['--no-header', '-bc'],
-                 coffee_executable: Optional[Union[str, Path]] = None,
+                 coffee_executable: Optional[StrOrPath] = None,
                  make_map: bool = False,
-                 coffee_concat_executable: Optional[Union[str, Path]] = None) -> None:
+                 coffee_concat_executable: Optional[StrOrPath] = None) -> None:
         if coffee_executable is None:
             coffee_executable = os_utils.which('coffee')
         if coffee_executable is None:
