@@ -1,7 +1,7 @@
 '''
 Enum generators for various languages.
 
-Copyright (c) 2015 - 2022 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
+Copyright (c) 2015 - 2024 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -24,25 +24,23 @@ SOFTWARE.
 
 import os
 from pathlib import Path
-from typing import List, Union
+from typing import List
 
-from buildtools import log, os_utils
+from ruamel.yaml import YAML
+
+from buildtools import os_utils
 from buildtools.maestro.base_target import SingleBuildTarget
-
-
 from buildtools.maestro.enumwriters.enumwriter import EnumWriter
-
-
 from buildtools.maestro.enumwriters.types import EnumDefinition
 
-from ruamel.yaml import YAML  # isort: skip
+from buildtools.types import StrOrPath  # isort: skip
 yaml = YAML(typ='safe', pure=True)
 
 
 class GenerateEnumTarget(SingleBuildTarget):
     BT_LABEL = 'ENUM'
 
-    def __init__(self, target: Union[str, Path], source: Union[str, Path], writer: EnumWriter, dependencies: List[str] = []) -> None:
+    def __init__(self, target: StrOrPath, source: StrOrPath, writer: EnumWriter, dependencies: List[str] = []) -> None:
         self.filename: Path = Path(source)
         self.outfile: Path = Path(target)
         self.writer: EnumWriter = writer
