@@ -2,7 +2,7 @@
 '''
 Twisted utilities
 
-Copyright (c) 2015 - 2022 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
+Copyright (c) 2015 - 2024 Rob "N3X15" Nelson <nexisentertainment@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -24,9 +24,11 @@ SOFTWARE.
 
 '''
 import os
+from pathlib import Path
 import sys
 import threading
 import time
+from typing import Any, List, Optional
 
 # package Twisted
 from twisted.internet import reactor
@@ -45,7 +47,7 @@ class _PipeReader(ProcessProtocol):
         self._cb_stderr = stderr_callback
         self._cb_exit = exit_callback
         self.process = process
-        self.linebreaks = linebreaks or ('\r','\n','')
+        self.linebreaks = linebreaks or ('\r', '\n', '')
 
         self.buf = {
             'stdout': '',
@@ -88,7 +90,7 @@ class _PipeReader(ProcessProtocol):
     def processEnded(self, reason):
         self._dump_to_callback('stderr', self._cb_stderr)
         self._dump_to_callback('stdout', self._cb_stdout)
-        self._asyncCommand.running=False
+        self._asyncCommand.running = False
         self._asyncCommand.exit_code = reason.value.exitCode
         self._cb_exit(self._asyncCommand, self._asyncCommand.exit_code)
 
@@ -111,9 +113,19 @@ class ReactorManager:
 
 
 class AsyncCommand(object):
-
-    def __init__(self, command, stdout=None, stderr=None, echo=False, env=None, PTY=False, refName=None, debug=False, globbify=True, linebreaks=None):
-        self.running=False
+    def __init__(self,
+                 command: List[str],
+                 stdout: Any = None,
+                 stderr: Any = None,
+                 echo: bool = False,
+                 env: Optional[os_utils.BuildEnv] = None,
+                 PTY: bool = False,
+                 refName: Optional[str] = None,
+                 debug: bool = False,
+                 globbify: bool = True,
+                 linebreaks: Optional[str] = None,
+                 executable: Optional[Path] = None) -> None:
+        self.running = False
         self.echo = echo
         self.command = command
         self.PTY = PTY
@@ -122,7 +134,7 @@ class AsyncCommand(object):
         self.linebreaks = linebreaks
 
         self.env = os_utils._cmd_handle_env(env)
-        self.command = os_utils._cmd_handle_args(command,globbify=globbify)
+        self.command = os_utils._cmd_handle_args(command, globbify=globbify)
 
         self.child = None
         self.refName = self.commandName = os.path.basename(self.command[0])
@@ -139,11 +151,12 @@ class AsyncCommand(object):
 
         self.pipe_reader = None
         self.debug = debug
+        self.executable = executable or Path(command[0])
 
     def default_exit_handler(self, ascmd, code):
         if code != 0:
             if code < 0:
-                strerr = '%s: Received signal %d' % (self.refName, 0-code)
+                strerr = '%s: Received signal %d' % (self.refName, 0 - code)
                 if code < -100:
                     strerr += ' (?!)'
                 self.log.error(strerr)
@@ -153,14 +166,14 @@ class AsyncCommand(object):
             self.log.info('%s has exited normally.', self.refName)
 
     def __addToBuffer(self, text, bufname, ascmd, logmethod):
-        #print(bufname, repr(text))
+        # print(bufname, repr(text))
         buf = getattr(self, bufname)
 
         for c in text:
             if c == '\r':
-                continue # Fuck you, you don't exist.
+                continue  # Fuck you, you don't exist.
             if c == '\n':
-                logmethod('[%s] %s', ascmd.refName, buf+c)
+                logmethod('[%s] %s', ascmd.refName, buf + c)
                 buf = ''
                 continue
             buf += c
@@ -177,11 +190,11 @@ class AsyncCommand(object):
             self.log.info('[ASYNC] $ "%s"', '" "'.join(self.command))
         pr = _PipeReader(self, self.child, self.stdout_callback, self.stderr_callback, self.exit_code_handler, self.linebreaks)
         pr.debug = self.debug
-        self.running=True
-        self.child = reactor.spawnProcess(pr, self.command[0], self.command[1:], env=self.env, usePTY=self.PTY)
+        self.running = True
+        self.child = reactor.spawnProcess(pr, self.executable, self.command[1:], env=self.env, usePTY=self.PTY)
         if self.child is None:
             self.log.error('Failed to start %r.', ' '.join(self.command))
-            self.running=False
+            self.running = False
             return False
         ReactorManager.Start()
         return True
@@ -199,7 +212,8 @@ class AsyncCommand(object):
     def IsRunning(self):
         return self.running
 
-def async_cmd(command, stdout=None, stderr=None, env=None):
-    ascmd = AsyncCommand(command, stdout=stdout, stderr=stderr, env=env)
+
+def async_cmd(command: List[str], stdout: Any = None, stderr: Any = None, env: Optional[os_utils.BuildEnv] = None, executable: Optional[Path] = None):
+    ascmd = AsyncCommand(command, stdout=stdout, stderr=stderr, env=env, executable=executable)
     ascmd.Start()
     return ascmd
